/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * Copyright (C) 2018 Daniel Mack
 */

#ifndef __DT_BINDINGS_MAX9485_CLK_H
#define __DT_BINDINGS_MAX9485_CLK_H

#define MAX9485_MCLKOUT	0
#define MAX9485_CLKOUT	1
#define MAX9485_CLKOUT1	2
#define MAX9485_CLKOUT2	3

#endif /* __DT_BINDINGS_MAX9485_CLK_H */
