/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __STE_CLK_AB8500_H__
#define __STE_CLK_AB8500_H__

#define AB8500_SYSCLK_BUF2	0
#define AB8500_SYSCLK_BUF3	1
#define AB8500_SYSCLK_BUF4	2
#define AB8500_SYSCLK_ULP	3
#define AB8500_SYSCLK_INT	4
#define AB8500_SYSCLK_AUDIO	5

#endif
