/* SPDX-License-Identifier: GPL-2.0+ */

#ifndef _DT_BINDINGS_ATMEL_MAXTOUCH_H
#define _DT_BINDINGS_ATMEL_MAXTOUCH_H

#define ATMEL_MXT_WAKEUP_NONE		0
#define ATMEL_MXT_WAKEUP_I2C_SCL	1
#define ATMEL_MXT_WAKEUP_GPIO		2

#endif /* _DT_BINDINGS_ATMEL_MAXTOUCH_H */
