/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * Copyright (C) 2014 Alexander Shiyan <shc_work@mail.ru>
 */

#ifndef __DT_BINDINGS_CLOCK_CLPS711X_H
#define __DT_BINDINGS_CLOCK_CLPS711X_H

#define CLPS711X_CLK_DUMMY	0
#define CLPS711X_CLK_CPU	1
#define CLPS711X_CLK_BUS	2
#define CLPS711X_CLK_PLL	3
#define CLPS711X_CLK_TIMERREF	4
#define CLPS711X_CLK_TIMER1	5
#define CLPS711X_CLK_TIMER2	6
#define CLPS711X_CLK_PWM	7
#define CLPS711X_CLK_SPIREF	8
#define CLPS711X_CLK_SPI	9
#define CLPS711X_CLK_UART	10
#define CLPS711X_CLK_TICK	11
#define CLPS711X_CLK_MAX	12

#endif
