/* SPDX-License-Identifier: (GPL-2.0 OR MIT) */
/*
 *  Copyright (C) 2020 Compass Electronics Group, LLC
 */

#ifndef __DT_BINDINGS_IMX8MN_POWER_H__
#define __DT_BINDINGS_IMX8MN_POWER_H__

#define IMX8MN_POWER_DOMAIN_HSIOMIX	0
#define IMX8MN_POWER_DOMAIN_OTG1	1
#define IMX8MN_POWER_DOMAIN_GPUMIX	2
#define IMX8MN_POWER_DOMAIN_DISPMIX	3
#define IMX8MN_POWER_DOMAIN_MIPI	4

#endif
