/* SPDX-License-Identifier: GPL-2.0+ */

#ifndef __DT_BINDINGS_BMIPS_BCM6362_PM_H
#define __DT_BINDINGS_BMIPS_BCM6362_PM_H

#define BCM6362_POWER_DOMAIN_SAR	0
#define BCM6362_POWER_DOMAIN_IPSEC	1
#define BCM6362_POWER_DOMAIN_MIPS	2
#define BCM6362_POWER_DOMAIN_DECT	3
#define BCM6362_POWER_DOMAIN_USBH	4
#define BCM6362_POWER_DOMAIN_USBD	5
#define BCM6362_POWER_DOMAIN_ROBOSW	6
#define BCM6362_POWER_DOMAIN_PCM	7
#define BCM6362_POWER_DOMAIN_PERIPH	8
#define BCM6362_POWER_DOMAIN_ADSL_PHY	9
#define BCM6362_POWER_DOMAIN_GMII_PADS	10
#define BCM6362_POWER_DOMAIN_FAP	11
#define BCM6362_POWER_DOMAIN_PCIE	12
#define BCM6362_POWER_DOMAIN_WLAN_PADS	13

#endif /* __DT_BINDINGS_BMIPS_BCM6362_PM_H */
